@@ -1,15 +1,16 @@
 package Plack::App::GitHub::WebHook;
 use strict;
+use warnings;
 use v5.10;
-use JSON qw(decode_json);
 
 use parent 'Plack::Component';
 use Plack::Util::Accessor qw(hook access app events safe);
 use Plack::Request;
 use Plack::Middleware::Access;
 use Carp qw(croak);
+use JSON qw(decode_json);
 
-our $VERSION = '0.5';
+our $VERSION = '0.6';
 
 sub prepare_app {
     my $self = shift;
@@ -36,12 +37,8 @@ sub prepare_app {
             rules => $self->access
         )
     );
-
-    $self->init; # TODO: not documented: remove?
 }
 
-sub init { }
-
 sub call {
     my ($self, $env) = @_;
     $self->app->($env);
@@ -55,32 +52,51 @@ sub call_granted {
     }
 
     my $req = Plack::Request->new($env);
-    my $event = $env->{'X_GITHUB_EVENT'};
+    my $event = $env->{'HTTP_X_GITHUB_EVENT'} // '';
+    my $delivery = $env->{'HTTP_X_GITHUB_DELIVERY'} // '';
     my $json;
     
     if ( !$self->events or grep { $event eq $_ } @{$self->events} ) {
-        $json = eval { decode_json $req->body_parameters->{payload} };
+        $json = eval { decode_json $req->content };
     }
 
     if (!$json) {
         return [400,['Content-Type'=>'text/plain','Content-Length'=>11],['Bad Request']];
     }
+    
+    my $log = sub {
+        $env->{'psgix.logger'}->({ level => $_[0], message => $_[1] });
+    };
+    my $logger = Plack::Util::inline_object(
+        debug => sub { $log->( debug => $_[0] ) },
+        info  => sub { $log->( info  => $_[0] ) },
+        warn  => sub { $log->( warn  => $_[0] ) },
+        error => sub { $log->( error => $_[0] ) },
+        fatal => sub { $log->( fatal => $_[0] ) },
+    );
 
-    if ( $self->receive($json) ) {
-        return [200,['Content-Type'=>'text/plain','Content-Length'=>2],['OK']];
+    my ($status, $message);
+    if ( $self->receive($json, $event, $delivery, $logger) ) {
+        ($status, $message) = (200,"OK");
     } else {
-        return [202,['Content-Type'=>'text/plain','Content-Length'=>8],['Accepted']];
+        ($status, $message) = (202,"Accepted");
     }
+
+    $message = ucfirst($event)." $message" if $self->events;
+
+    [ $status,
+        [ 'Content-Type' => 'text/plain', 'Content-Length' => length $message ],
+        [$message] ];
 }
 
 sub receive {
-    my ($self, $payload) = @_;
+    my $self = shift;
 
     foreach my $hook (@{$self->{hook}}) {
         if ($self->safe) {
-            return unless eval { $hook->($payload) } and !$@;
+            return unless eval { $hook->(@_) } and !$@;
         } else {
-            return unless $hook->($payload);
+            return unless $hook->(@_);
         }
     } 
 
@@ -94,6 +110,16 @@ __END__
 
 Plack::App::GitHub::WebHook - GitHub WebHook receiver as Plack application
 
+=begin markdown
+
+# STATUS
+
+[![Build Status](https://travis-ci.org/nichtich/Plack-App-GitHub-WebHook.png)](https://travis-ci.org/nichtich/Plack-App-GitHub-WebHook)
+[![Coverage Status](https://coveralls.io/repos/nichtich/Plack-App-GitHub-WebHook/badge.png?branch=master)](https://coveralls.io/r/nichtich/Plack-App-GitHub-WebHook?branch=master)
+[![Kwalitee Score](http://cpants.cpanauthors.org/dist/Plack-App-GitHub-WebHook.png)](http://cpants.cpanauthors.org/dist/Plack-App-GitHub-WebHook)
+
+=end markdown
+
 =head1 SYNOPSIS
 
 =head2 Basic usage
@@ -155,23 +181,31 @@ By default access is restricted to known GitHub WebHook IPs.
 =head2 Synchronize with a GitHub repository
 
 The following application automatically pulls the master branch of a GitHub
-repository into a local working directory C<$work_tree>.
+repository into a local working directory.
 
-    use Git::Repository
+    use Git::Repository;
     use Plack::App::GitHub::WebHook;
 
+    my $branch = "master;
+    my $work_tree = "/some/path";
+
     Plack::App::GitHub::WebHook->new(
-        events => ['pull'],
+        events => ['push','ping'],
         safe => 1,
         hook => [
-            sub { $_[0]->{ref} eq 'refs/heads/master' },
+            sub { 
+                my ($payload, $method) = @_;
+                $method eq 'ping' or $payload->{ref} eq "refs/heads/$branch";
+            },
             sub {
+                my ($payload, $method) = @_;
+                return 1 if $method eq 'ping'; 
                 if ( -d "$work_tree/.git") {
                     Git::Repository->new( work_tree => $work_tree )
-                                   ->run(qw(pull origin master));
+                                   ->run( 'pull', origin => $branch );
                 } else {
-                    my $origin = $_[0]->{repository}->{clone_url};
-                    Git::Repository->run( 'clone', $origin, $work_tree );
+                    my $origin = $payload->{repository}->{clone_url};
+                    Git::Repository->run( clone => $origin, -b => $branch, $work_tree );
                 }
                 1;
             },
@@ -222,10 +256,11 @@ This module requires at least Perl 5.10.
 =item hook
 
 A code reference or an array of code references with tasks that are executed on
-an incoming webhook.  Each task gets passed the encoded payload. If the task
-returns a true value, next the task is called or HTTP status code 200 is
-returned. Information can be passed from one task to the next by modifying the
-payload. 
+an incoming webhook.  Each task gets passed the encoded payload, the
+L<event|https://developer.github.com/webhooks/#events> and the unique delivery
+ID.  If the task returns a true value, next the task is called or HTTP status
+code 200 is returned.  Information can be passed from one task to the next by
+modifying the payload. 
 
 If a task returns a false value or if no task was given, HTTP status code 202
 is returned immediately. This mechanism can be used for conditional hooks or to
@@ -253,6 +288,29 @@ expected to be send with the C<X-GitHub-Event> header (e.g. C<['pull']>).
 
 =back
 
+=head1 DEPLOYMENT
+
+Many deployment methods exist. An easy option might be to use Apache webserver
+with mod_cgi and L<Plack::Handler::CGI>. First install Apache, Plack and
+Plack::App::GitHub::WebHook:
+
+    sudo apt-get install apache2
+    sudo apt-get install cpanminus libplack-perl
+    sudo cpanm Plack::App::GitHub::WebHook
+
+Then add this section to C</etc/apache2/sites-enabled/default> (or another host
+configuration) and restart apache afterwards (C<sudo service apache2 restart>):
+
+    <Directory /var/www/webhooks>
+       Options +ExecCGI -Indexes +SymLinksIfOwnerMatch
+       AddHandler cgi-script .cgi
+    </Directory>
+
+You can now put webhook applications in directory C</var/www/webhooks> as long
+as they are executable, have file extension C<.cgi> and shebang line
+C<#!/usr/bin/env plackup>. You might further want to run webhooks scripts as
+another user instead of C<www-data> by using Apache module SuExec.
+
 =head1 SEE ALSO
 
 =over
@@ -289,20 +347,9 @@ L<App::GitHubWebhooks2Ikachan> is an application that also receives GitHub WebHo
 
 =back
 
-=begin markdown
+=head1 COPYRIGHT AND LICENSE
 
-# STATUS
-
-[![Build Status](https://travis-ci.org/nichtich/Plack-App-GitHub-WebHook.png)](https://travis-ci.org/nichtich/Plack-App-GitHub-WebHook)
-[![Coverage Status](https://coveralls.io/repos/nichtich/Plack-App-GitHub-WebHook/badge.png?branch=master)](https://coveralls.io/r/nichtich/Plack-App-GitHub-WebHook?branch=master)
-
-=end markdown
-
-=head1 AUTHOR
-
-Jakob Voss C<< <jakob.voss@gbv.de> >>
-
-=head1 LICENSE
+Copyright Jakob Voss, 2014-
 
 This library is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
